{ TDSGroupBox is an enhanced GroupBox component that allows you to display a
  drop-shadow, change the type of border and draw an enhanced caption on the
  control. The component is very easy to use, and possesses only five extra
  properties:

  GroupType        - describes the type of GroupBox displayed, and can be either
                     drop-shadow, plain, Raised3D, Raised3DShadow;
  EnhancedCaption  - produces an enhanced caption;
  ShadowColor      - the color of the dropped shadow;
  XOffSet          - offset in the X direction;
  YOffSet          - offset in the Y direction; }

{ Written by N.Hollinworth 10th May 1999 }

unit DSGroupBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

const
  OffSet = 5;

type
  TGroupType = (Plain, DropShadow, Raised3D, Raised3DShadow);
  TDSGroupBox = class(TGroupBox)
  private
    FEnhanceCaption: boolean;
    FShadowColor: TColor;
    FGroupType: TGroupType;
    FXOffSet: integer;
    FYOffSet: integer;
    procedure SetShadowColor(const Value: TColor);
    procedure SetGroupType(const Value: TGroupType);
    procedure SetXOffSet(const Value: integer);
    procedure SetYOffSet(const Value: integer);
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure SetEnhanceCaption(const Value: boolean);
  protected
    procedure Paint; override;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property EnhanceCaption: boolean read FEnhanceCaption write SetEnhanceCaption default false;
    property GroupType: TGroupType read FGroupType write SetGroupType default Plain;
    property XOffSet: integer read FXOffSet write SetXOffSet default OffSet;
    property YOffSet: integer read FYOffSet write SetYOffSet default OffSet;
    property ShadowColor: TColor read FShadowColor write SetShadowColor;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TDSGroupBox]);
end;

{ TDSGroupBox }

{ If the parent's color changes, then repaint the control }
procedure TDSGroupBox.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TDSGroupBox.Create(AOwner: TComponent);
begin
  inherited;
  FEnhanceCaption := false;
  FGroupType := Plain;
  FShadowColor := clBlack;
  FXOffSet := OffSet;
  FYOffSet := OffSet;
end;

procedure TDSGroupBox.Paint;
var
  H, OldBkMode: Integer;
  R: TRect;
  Flags: Longint;
  FontColor: TColor;
  FaceColor: TColor;
  Shadow: TRect;

  { Utility: set the dimensions of the rectangle for painting }
  function SetRect(FXOffSet, FYOffSet: integer): TRect;
  begin
    if FXOffSet < 0 then
      Result := Rect(-FXOffSet, H div 2 - 1, Width, Height - FYOffSet);

    if FYOffSet < 0 then
      Result := Rect(0, H div 2 - 1 - FYOffSet, Width - FXOffSet, Height);

    if (FXOffSet < 0) and (FYOffSet < 0) then
      Result := Rect(-FXOffSet, H div 2 - 1 - FYOffSet, Width, Height);

    if (FXOffSet >= 0) and (FYOffSet >= 0) then
      Result := Rect(0, H div 2 - 1, Width - FXOffSet, Height - FYOffSet);
  end;

begin
  with Canvas do begin
    Font := Self.Font;
    FontColor := Font.Color;
    FaceColor := Color;
    H := TextHeight('0');
    R := GetClientRect;

    // Set the colour of the backdrop before any other painting
    Canvas.Brush.Color := Parent.Brush.Color;
    Canvas.FillRect(R);

    case FGroupType of
    DropShadow:
    begin
      Shadow.Left := R.Left + FXOffSet;
      Shadow.Top := R.Top + FYOffSet + (H div 2 - 1);

      if FXOffSet < 0 then
        Shadow.Right := R.Right + FXOffSet
      else
        Shadow.Right := R.Right;

      if FYOffSet < 0 then
        Shadow.Bottom := R.Bottom + FYOffSet
      else
        Shadow.Bottom := R.Bottom;

      Brush.Color := FShadowColor;
      FillRect(Shadow);
      R := SetRect(FXOffSet, FYOffSet);
    end;
    Plain:
    begin
      R := Rect(0, H div 2 - 1, Width, Height);
    end;
    Raised3D:
    begin
      R := Rect(0, H div 2 - 1, Width, Height);
      Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
    end;
    Raised3DShadow:
    begin
      Shadow.Left := R.Left + FXOffSet;
      Shadow.Top := R.Top + FYOffSet + (H div 2);

      if FXOffSet < 0 then
        Shadow.Right := R.Right + FXOffSet
      else
        Shadow.Right := R.Right;

      if FYOffSet < 0 then
        Shadow.Bottom := R.Bottom + FYOffSet
      else
        Shadow.Bottom := R.Bottom;
        
      Brush.Color := FShadowColor;
      FillRect(Shadow);
      R := SetRect(FXOffSet, FYOffSet);
      Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
    end;
    end; { case }

    // Paint the face of the control
    Brush.Color := FaceColor;
    FillRect(R);

    // Paint border
    if FGroupType in [Plain, DropShadow] then begin
      if Ctl3D then begin
        Inc(R.Left);
        Inc(R.Top);
        Brush.Color := clBtnHighlight;
        FrameRect(R);
        OffsetRect(R, -1, -1);
        Brush.Color := clBtnShadow;
      end else
        Brush.Color := clWindowFrame;
      FrameRect(R);
    end;

    // Draw text
    if Text <> '' then begin
      if not UseRightToLeftAlignment then
        if FYOffSet < 0 then
          R := Rect(8, -FYOffSet, 0, H)
        else
          R := Rect(8, 0, 0, H)
      else
        R := Rect(R.Right - Canvas.TextWidth(Text) - 8, FYOffSet - (H div 2 - 1), 0, H);

      Flags := DrawTextBiDiModeFlags(DT_SINGLELINE);
      if FEnhanceCaption then begin
        Dec(R.Left);
        OldBkMode := SetBkMode(Handle, TRANSPARENT);
        Font.Color := clWhite;
        DrawText(Handle, PChar(Text), Length(Text), R, Flags or DT_CALCRECT);
        Brush.Color := Color;
        DrawText(Handle, PChar(Text), Length(Text), R, Flags);
        Brush.Style := bsClear;
        Font.Color := FontColor;
        Inc(R.Left);
        Inc(R.Top);
        DrawText(Handle, PChar(Text), Length(Text), R, Flags or DT_CALCRECT);
        DrawText(Handle, PChar(Text), Length(Text), R, Flags);
        SetBkMode(Handle, OldBkMode);
      end
      else begin
        OldBkMode := SetBkMode(Handle, TRANSPARENT);
        DrawText(Handle, PChar(Text), Length(Text), R, Flags or DT_CALCRECT);
        Brush.Color := Color;
        DrawText(Handle, PChar(Text), Length(Text), R, Flags);
        SetBkMode(Handle, OldBkMode);
      end;
    end;
  end;
end;

procedure TDSGroupBox.SetEnhanceCaption(const Value: boolean);
begin
  FEnhanceCaption := Value;
  invalidate;
end;

procedure TDSGroupBox.SetGroupType(const Value: TGroupType);
begin
  FGroupType := Value;
  Invalidate;
end;

procedure TDSGroupBox.SetShadowColor(const Value: TColor);
begin
  FShadowColor := Value;
  Invalidate;
end;

procedure TDSGroupBox.SetXOffSet(const Value: integer);
begin
  FXOffSet := Value;
  Invalidate;
end;

procedure TDSGroupBox.SetYOffSet(const Value: integer);
begin
  FYOffSet := Value;
  Invalidate;
end;

end.
